#include <QLocale>
#include <QSqlQuery>
#include <QThread>
#include <QVariant>
#include <QVector>

#include "account.h"
#include "db.h"

Account::Account(QThread *thread) :
	m_activeIndex(-1)
{
	this->moveToThread(thread);
	QObject::connect(thread,	&QThread::started,	this,	&Account::process);
	QObject::connect(this,		&Account::finished,	thread,	&QThread::quit);
	QObject::connect(this,		&Account::finished,	this,	&Account::deleteLater);
	QObject::connect(thread,	&QThread::finished,	thread,	&QThread::deleteLater);
}

Account::~Account() {
	m_mutex.lock();
	m_data->clear();
	delete m_data;
	m_mutex.unlock();
}

QString Account::formatCurrency(qint64 amount) {
	QLocale locale(QLocale::Dutch);
	QString amountStr = locale.toCurrencyString(amount / FACTOR, " ").trimmed();
	amountStr.prepend(locale.currencySymbol(QLocale::CurrencySymbol));

	return amountStr;
}

void Account::process() {
	m_mutex.lock();
	m_data = new QVector<Data>;
	m_mutex.unlock();
}

void Account::getAccounts(int userId) {
	m_mutex.lock();
	m_data->clear();
	m_mutex.unlock();

	QSqlQuery query = Db::q("SELECT account.id, number, name, amount FROM user \
INNER JOIN \
	user_has_account ON user.id = user_has_account.user_id \
INNER JOIN \
    account ON account.id = user_has_account.account_id \
WHERE user.id=?", {userId});

	if (query.exec()) {
		while (query.next()) {
			Data tmp = {
				query.value("id").toInt(),
				query.value("number").toString(),
				query.value("name").toString(),
				query.value("amount").toLongLong(),
			};
			m_mutex.lock();
			m_data->append(tmp);
			m_mutex.unlock();
		}

		m_mutex.lock();
		for (int i = 0; i < m_data->size(); i++) {
			printf("Account, id: %d\n", m_data->value(i).id);
		}
		m_mutex.unlock();
	}
}

void Account::updateActiveIndex() {
	if (m_activeIndex != -1) {
		Data tmp = getAccount(getAccount(m_activeIndex).id, true);
		if (tmp.id != 0) {
			m_mutex.lock();
			m_data->replace(m_activeIndex, tmp);
			m_mutex.unlock();

			emit emitLoadBalance(formatCurrency(tmp.amount));
		}
	}
}

void Account::clearData() {
    m_mutex.lock();
    m_data->clear();
    m_mutex.unlock();
}

int Account::getSize() {
	QMutexLocker locker(&m_mutex);
	return m_data->size();
}

Account::Data Account::getAccount(int index) {
	QMutexLocker locker(&m_mutex);
	if (index > -1) {
		return m_data->value(index);
	}
	else if (m_activeIndex > -1) {
		return m_data->value(m_activeIndex);
	}

	return {};
}

Account::Data Account::getAccount(int id, bool fromDatabase) {
	QSqlQuery query = Db::q("SELECT `id`, `number`, `name`, `amount` \
			FROM `account` WHERE `id`=?", {id});
	if (query.exec() && query.size() == 1 && query.next()) {
		return {
			query.value("id").toInt(),
			query.value("number").toString(),
			query.value("name").toString(),
			query.value("amount").toLongLong(),
		};
	}

	return {};
}

Account::Data Account::getAccount(QString number) {
	QSqlQuery query = Db::q("SELECT `id`, `number`, `name`, `amount` \
			FROM `account` WHERE `number`=?", {number});
	if (query.exec() && query.size() == 1 && query.next()) {
		return Data {
			query.value("id").toInt(),
			query.value("number").toString(),
			query.value("name").toString(),
			query.value("amount").toLongLong(),
		};
	}

	return {};
}

int Account::getActiveIndex() {
	QMutexLocker locker(&m_mutex);
	return m_activeIndex;
}

void Account::setActiveIndex(int index) {
	QMutexLocker locker(&m_mutex);
	m_activeIndex = index;
}
