#include <QThread>

#include "account.h"
#include "card.h"
#include "log.h"
#include "serialcommand.h"
#include "transaction.h"
#include "ui_mainwindow.h"
#include "user.h"

SerialCommand::SerialCommand(QThread *thread, Card *card, User *user,
		Account *account, Log *log, Transaction *transaction) :
	m_cardDetected(false),
	m_cardId(""),
	m_pin(""),
	m_transferAccountSelected(-1),
	m_card(card),
	m_user(user),
	m_account(account),
	m_log(log),
	m_transaction(transaction)
{
	this->moveToThread(thread);
	QObject::connect(thread,	&QThread::started,			this,	&SerialCommand::process);
	QObject::connect(this,		&SerialCommand::finished,	thread,	&QThread::quit);
	QObject::connect(this,		&SerialCommand::finished,	this,	&SerialCommand::deleteLater);
	QObject::connect(thread,	&QThread::finished,			thread,	&QThread::deleteLater);
}

SerialCommand::~SerialCommand() {

}

void SerialCommand::setup(MainWindow *mainWindow) {
	m_mainWindow = mainWindow;

	// Ui connects
	connect(this, &SerialCommand::emitWindowState, m_mainWindow, &MainWindow::setWindowState);

	connect(this, &SerialCommand::cardDetected, m_mainWindow->getUi()->loginPage, &LoginPage::cardDetected);
	connect(this, &SerialCommand::pinEntered, m_mainWindow->getUi()->loginPage, &LoginPage::pinEntered);
	connect(this, &SerialCommand::emitNumberEntered, m_mainWindow->getUi()->withdrawPage, &WithdrawPage::numberEntered);
	connect(this, &SerialCommand::emitNumberEntered, m_mainWindow->getUi()->transferPage, &TransferPage::numberEntered);
	connect(this, &SerialCommand::emitSelectAccount, m_mainWindow->getUi()->transferPage, &TransferPage::selectAccount);
	connect(this, &SerialCommand::emitSelectElement, m_mainWindow->getUi()->transferPage, &TransferPage::selectElement);

	// Db connects
	connect(this, &SerialCommand::emitNewLog, m_log, &Log::newLog);
	connect(this, &SerialCommand::emitTransfer, m_transaction, &Transaction::transfer);
	connect(m_card, &Card::emitNewLog, m_log, &Log::newLog);
	connect(m_transaction, &Transaction::emitNewLog, m_log, &Log::newLog);
}

void SerialCommand::process() {
}

void SerialCommand::processData(QByteArray data) {
	const char *command = data.data();
#ifdef DEBUG
	// printf("SerialCommand, %s\n", command);
#endif

	int type = command[0];
	if (type == SerialCommand::CARD_DETECTED) {
		if (command[1] == 1) {
			m_cardDetected = true;
			emit cardDetected(true);
		}
		else if (command[1] == 0) {
			emit emitNewLog(Log::LOGOUT, "", 1, "", "");

			m_card->setId("");
			m_user->setId(0);
			m_user->setFirstName("");
			m_user->setLastName("");
			m_user->setRole(0);
			m_account->setActiveIndex(-1);
			m_account->clearData();

			m_cardDetected = false;
			m_cardId = "";
			m_pin = "";
			m_transferAccountSelected = -1;
			emit cardDetected(false);
			emit emitWindowState(MainWindow::LOGIN_PAGE);
		}
	}
	else if (type == SerialCommand::CARD_ID) {
		if (data.size() > 3) {
			command++;
			m_cardId = QString(QByteArray(command).toHex()).toUpper();

			for (int i = m_cardId.size(); i < 12; i++) {
				m_cardId.append("0");
			}
#ifdef DEBUG
			printf("SerialCommand, %s\n", m_cardId.toUtf8().data());
#endif
		}
	}
	else if (type == SerialCommand::PIN) {
#ifdef DEBUG
		printf("SerialCommand, %c\n", command[1]);
#endif
		if (m_mainWindow->getWindowState() == MainWindow::LOGIN_PAGE) {
			handleLoginPage(command[1]);
		}
		else if (m_mainWindow->getWindowState() == MainWindow::HOME_PAGE) {
			handleHomePage(command[1]);
		}
		else if (m_mainWindow->getWindowState() == MainWindow::ACCOUNT_PAGE) {
			handleAccountPage(command[1]);
		}
		else if (m_mainWindow->getWindowState() == MainWindow::BALANCE_PAGE) {
			handleBalancePage(command[1]);
		}
		else if (m_mainWindow->getWindowState() == MainWindow::WITHDRAW_PAGE) {
			handleWithdrawPage(command[1]);
		}
		else if (m_mainWindow->getWindowState() == MainWindow::TRANSFER_PAGE) {
			handleTransferPage(command[1]);
		}
	}
}

void SerialCommand::handleLoginPage(char input) {
	if (m_cardDetected && !m_cardId.isEmpty()) {
		// 0-9
		if (input >= 48 && input <= 57) {
			m_pin.append(input);
			emit pinEntered(m_pin.size());
		}
		else if (input == 'A') {
			// Abort
			m_pin.clear();
			emit pinEntered(0);
		}
		else if (input == 'B') {
			// Correct
			m_pin.chop(1);
			emit pinEntered(m_pin.size());
		}
		else if (input == 'C') {
			// Info
#ifdef DEBUG
			printf("SerialCommand, pin: %s\n", m_pin.toUtf8().data());
#endif
		}
		else if (input == 'D') {
			// Confirm
			emit pinEntered(0);
			attemptLogin();
			m_pin.clear();
		}
	}
}

void SerialCommand::handleHomePage(char input) {
	m_pin.clear();

	if (input == '1') {
		emit emitWindowState(MainWindow::ACCOUNT_PAGE);
	}
	else if (input == '2') {
		if (m_account->getActiveIndex() >= 0) {
			emit emitWindowState(MainWindow::BALANCE_PAGE);
		}
	}
	else if (input == '3') {
		if (m_account->getActiveIndex() >= 0) {
			emit emitWindowState(MainWindow::WITHDRAW_PAGE);
		}
	}
	else if (input == '4') {
		if (m_account->getActiveIndex() >= 0) {
			emit emitWindowState(MainWindow::TRANSFER_PAGE);
		}
	}
}

void SerialCommand::handleAccountPage(char input) {
	// 0-9
	if (input >= 48 && input <= 57) {
		int index = input - 48 - 1;
		if (index >= 0 && index < m_account->getSize()) {
			m_account->setActiveIndex(index);
			emit emitWindowState(MainWindow::HOME_PAGE);
		}
	}
	else if (input == 'A') {
		emit emitWindowState(MainWindow::HOME_PAGE);
	}
	else if (input == 'B') {
	}
	else if (input == 'C') {
	}
	else if (input == 'D') {
	}
}

void SerialCommand::handleBalancePage(char input) {
	if (input == 'A') {
		emit emitWindowState(MainWindow::HOME_PAGE);
	}
	else if (input == 'B') {
	}
	else if (input == 'C') {
	}
	else if (input == 'D') {
	}
}

void SerialCommand::handleWithdrawPage(char input) {
	// 0-9
	if (input >= 48 && input <= 57) {
		// Append
		m_pin.append(input);
		emit emitNumberEntered(m_pin.toLongLong());
	}
	if (input == 'A') {
		// Abort
		emit emitWindowState(MainWindow::HOME_PAGE);
	}
	else if (input == 'B') {
		// Correct
		m_pin.chop(1);
		emit emitNumberEntered(m_pin.toLongLong());
	}
	else if (input == 'C') {
	}
	else if (input == 'D') {
		// Confirm
		emit emitTransfer(m_pin.toLongLong(), "");
		m_pin.clear();
	}
}

void SerialCommand::handleTransferPage(char input) {
	// 0-9
	if (input >= 48 && input <= 57) {
		if (m_transferAccountSelected == -1) {
			// Account selection
			int intInput = input - 48 - 1;
			if (intInput >= 0 && intInput <= m_account->getSize()) {
				m_pin = QString::number(intInput);
				emit emitSelectAccount(intInput);
			}
		}
		else {
			// Amount selection
			m_pin.append(input);
			emit emitNumberEntered(m_pin.toLongLong());
		}
	}
	if (input == 'A') {
		// Abort
		if (m_transferAccountSelected == -1) {
			emit emitWindowState(MainWindow::HOME_PAGE);
		}
		else {
			m_transferAccountSelected = -1;
			emit emitSelectElement(false);
		}
	}
	else if (input == 'B') {
		// Correct
		if (m_transferAccountSelected >= 0) {
			m_pin.chop(1);
			emit emitNumberEntered(m_pin.toLongLong());
		}
	}
	else if (input == 'C') {
	}
	else if (input == 'D') {
		// Confirm
		if (m_transferAccountSelected == -1) {
			m_transferAccountSelected = m_pin.toInt();
			m_pin.clear();
			emit emitSelectElement(true);
		}
		else {
			QString number = m_account->getAccount(m_transferAccountSelected).number;
			emit emitTransfer(m_pin.toLongLong(), number);
			m_pin.clear();
			emit emitNumberEntered(0);
		}
	}
}

void SerialCommand::attemptLogin() {
	if (!m_pin.isEmpty() && m_card->getId().isEmpty() && m_user->getId() == 0) {
		emit loginCard(m_cardId, m_pin);

		int maxCheckTime = 2000;
		while (maxCheckTime > 0 &&
				m_card->getId().isEmpty() && m_user->getId() == 0) {
			QThread::msleep(5);
			maxCheckTime -= 5;
		}
	}
}
